@@ -9,6 +9,8 @@
 # with Bazaar or `bzr`, that was formerly known as Bazaar-NG;
 # it cannot access `bar` repositories, but they were never used very much.
 
+from __future__ import annotations
+
 import os
 
 from mercurial.i18n import _
@@ -23,24 +25,33 @@ from . import common
 # these do not work with demandimport, blacklist
 demandimport.IGNORES.update(
     [
-        b'breezy.transactions',
-        b'breezy.urlutils',
-        b'ElementPath',
+        'breezy.transactions',
+        'breezy.urlutils',
+        'ElementPath',
     ]
 )
 
 try:
     # bazaar imports
+    # pytype: disable=import-error
     import breezy.bzr.bzrdir
     import breezy.errors
     import breezy.revision
     import breezy.revisionspec
 
+    # pytype: enable=import-error
+
     bzrdir = breezy.bzr.bzrdir
     errors = breezy.errors
     revision = breezy.revision
     revisionspec = breezy.revisionspec
     revisionspec.RevisionSpec
+
+    try:
+        # brz 3.3.0 (revno: 7614.2.2)
+        from breezy.transport import NoSuchFile
+    except ImportError:
+        from breezy.errors import NoSuchFile
 except ImportError:
     pass
 
@@ -51,7 +62,7 @@ class bzr_source(common.converter_source):
     """Reads Bazaar repositories by using the Bazaar Python libraries"""
 
     def __init__(self, ui, repotype, path, revs=None):
-        super(bzr_source, self).__init__(ui, repotype, path, revs=revs)
+        super().__init__(ui, repotype, path, revs=revs)
 
         if not os.path.exists(os.path.join(path, b'.bzr')):
             raise common.NoRepo(
@@ -150,7 +161,7 @@ class bzr_source(common.converter_source):
 
         try:
             kind = revtree.kind(name)
-        except breezy.errors.NoSuchFile:
+        except NoSuchFile:
             return None, None
         if kind not in supportedkinds:
             # the file is not available anymore - was deleted
@@ -252,7 +263,7 @@ class bzr_source(common.converter_source):
             paths = change.path
             kind = change.kind
             executable = change.executable
-            if paths[0] == u'' or paths[1] == u'':
+            if paths[0] == '' or paths[1] == '':
                 # ignore changes to tree root
                 continue
 
@@ -321,7 +332,7 @@ class bzr_source(common.converter_source):
                 changes.append((path, revid))
 
             # populate the mode cache
-            kind, executable = [e[1] for e in (kind, executable)]
+            kind, executable = (e[1] for e in (kind, executable))
             mode = (executable and b'x') or (kind == 'symlink' and b'l') or b''
             self._modecache[(topath, revid)] = mode
             changes.append((topath, revid))
