@@ -5,6 +5,7 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
+from __future__ import annotations
 
 import io
 import os
@@ -126,15 +127,13 @@ def peersetup(ui, peer):
 
         def _sendrequest(self, command, args, **opts):
             self._updatecallstreamopts(command, args)
-            return super(remotefilepeer, self)._sendrequest(
-                command, args, **opts
-            )
+            return super()._sendrequest(command, args, **opts)
 
         def _callstream(self, command, **opts):
-            supertype = super(remotefilepeer, self)
+            supertype = super()
             if not hasattr(supertype, '_sendrequest'):
                 self._updatecallstreamopts(command, pycompat.byteskwargs(opts))
-            return super(remotefilepeer, self)._callstream(command, **opts)
+            return super()._callstream(command, **opts)
 
     peer.__class__ = remotefilepeer
 
@@ -192,7 +191,7 @@ class cacheconnection:
                 self.pipei.write(request)
                 if flush:
                     self.pipei.flush()
-            except IOError:
+            except OSError:
                 self.close()
 
     def receiveline(self):
@@ -202,7 +201,7 @@ class cacheconnection:
             result = self.pipeo.readline()[:-1]
             if not result:
                 self.close()
-        except IOError:
+        except OSError:
             self.close()
 
         return result
