@@ -145,6 +145,7 @@ future, dropping the stream may become an option for channel we do not care to
 preserve.
 """
 
+from __future__ import annotations
 
 import collections
 import errno
@@ -153,6 +154,7 @@ import re
 import string
 import struct
 import sys
+import typing
 
 from .i18n import _
 from .node import (
@@ -181,6 +183,14 @@ from .utils import (
 )
 from .interfaces import repository
 
+if typing.TYPE_CHECKING:
+    from typing import (
+        Iterator,
+        Union,
+    )
+
+    Capabilities = dict[bytes, Union[list[bytes], tuple[bytes, ...]]]
+
 urlerr = util.urlerr
 urlreq = util.urlreq
 
@@ -602,7 +612,7 @@ def _processpart(op, part):
             )
 
 
-def decodecaps(blob):
+def decodecaps(blob: bytes) -> Capabilities:
     """decode a bundle2 caps bytes blob into a dictionary
 
     The blob is a list of capabilities (one per line)
@@ -662,11 +672,14 @@ class bundle20:
 
     _magicstring = b'HG20'
 
-    def __init__(self, ui, capabilities=()):
+    def __init__(self, ui, capabilities: Capabilities | None = None):
+        if capabilities is None:
+            capabilities = {}
+
         self.ui = ui
         self._params = []
         self._parts = []
-        self.capabilities = dict(capabilities)
+        self.capabilities: Capabilities = dict(capabilities)
         self._compengine = util.compengines.forbundletype(b'UN')
         self._compopts = None
         # If compression is being handled by a consumer of the raw
@@ -723,7 +736,7 @@ class bundle20:
         return part
 
     # methods used to generate the bundle2 stream
-    def getchunks(self):
+    def getchunks(self) -> Iterator[bytes]:
         if self.ui.debugflag:
             msg = [b'bundle2-output-bundle: "%s",' % self._magicstring]
             if self._params:
@@ -737,10 +750,9 @@ class bundle20:
         yield _pack(_fstreamparamsize, len(param))
         if param:
             yield param
-        for chunk in self._compengine.compressstream(
+        yield from self._compengine.compressstream(
             self._getcorechunk(), self._compopts
-        ):
-            yield chunk
+        )
 
     def _paramchunk(self):
         """return a encoded version of all stream parameters"""
@@ -760,8 +772,7 @@ class bundle20:
         outdebug(self.ui, b'start of parts')
         for part in self._parts:
             outdebug(self.ui, b'bundle part: "%s"' % part.type)
-            for chunk in part.getchunks(ui=self.ui):
-                yield chunk
+            yield from part.getchunks(ui=self.ui)
         outdebug(self.ui, b'end of bundle')
         yield _pack(_fpartheadersize, 0)
 
@@ -837,7 +848,7 @@ class unbundle20(unpackermixin):
         self.ui = ui
         self._compengine = util.compengines.forbundletype(b'UN')
         self._compressed = None
-        super(unbundle20, self).__init__(fp)
+        super().__init__(fp)
 
     @util.propertycache
     def params(self):
@@ -854,7 +865,9 @@ class unbundle20(unpackermixin):
             params = self._processallparams(params)
         return params
 
-    def _processallparams(self, paramsblock):
+    def _processallparams(
+        self, paramsblock: bytes
+    ) -> util.sortdict[bytes, bytes]:
         """ """
         params = util.sortdict()
         for p in paramsblock.split(b' '):
@@ -924,7 +937,16 @@ class unbundle20(unpackermixin):
             yield _pack(_fstreamparamsize, paramssize)
         # From there, payload might need to be decompressed
         self._fp = self._compengine.decompressorreader(self._fp)
-        emptycount = 0
+
+        # We usually wait for empty terminators: a bundle part terminator
+        # followed by a bundle terminator.
+        #
+        # Since the empty bundle has no parts,
+        # bundle part terminator never comes.
+        #
+        # So we boostrap this detection with am empty part, which the first
+        # piece of part will reset.
+        emptycount = 1
         while emptycount < 2:
             # so we can brainlessly loop
             assert _fpartheadersize == _fpayloadsize
@@ -1253,7 +1275,7 @@ class interrupthandler(unpackermixin):
     Part processed in this manner only have access to a ui object,"""
 
     def __init__(self, ui, fp):
-        super(interrupthandler, self).__init__(fp)
+        super().__init__(fp)
         self.ui = ui
 
     def _readpartheader(self):
@@ -1271,7 +1293,6 @@ class interrupthandler(unpackermixin):
         return None
 
     def __call__(self):
-
         self.ui.debug(
             b'bundle2-input-stream-interrupt: opening out of band context\n'
         )
@@ -1375,7 +1396,7 @@ class unbundlepart(unpackermixin):
     """a bundle part read from a bundle"""
 
     def __init__(self, ui, header, fp):
-        super(unbundlepart, self).__init__(fp)
+        super().__init__(fp)
         self._seekable = hasattr(fp, 'seek') and hasattr(fp, 'tell')
         self.ui = ui
         # unbundle state attr
@@ -1453,6 +1474,11 @@ class unbundlepart(unpackermixin):
         # we read the data, tell it
         self._initialized = True
 
+    def __iter__(self):
+        for chunk in self._payloadstream:
+            self._pos += len(chunk)
+            yield chunk
+
     def _payloadchunks(self):
         """Generator of decoded chunks in the payload."""
         return decodepayloadchunks(self.ui, self._fp)
@@ -1488,6 +1514,10 @@ class unbundlepart(unpackermixin):
             self.consumed = True
         return data
 
+    def tell(self) -> int:
+        """the amount of byte read so far in the part"""
+        return self._payloadstream.tell()
+
 
 class seekableunbundlepart(unbundlepart):
     """A bundle2 part in a bundle that is seekable.
@@ -1511,7 +1541,7 @@ class seekableunbundlepart(unbundlepart):
         # (payload, file) offsets for chunk starts.
         self._chunkindex = []
 
-        super(seekableunbundlepart, self).__init__(ui, header, fp)
+        super().__init__(ui, header, fp)
 
     def _payloadchunks(self, chunknum=0):
         '''seek to specified chunk and start yielding data'''
@@ -1602,7 +1632,7 @@ class seekableunbundlepart(unbundlepart):
         if self._seekable:
             try:
                 return self._fp.tell()
-            except IOError as e:
+            except OSError as e:
                 if e.errno == errno.ESPIPE:
                     self._seekable = False
                 else:
@@ -1612,7 +1642,7 @@ class seekableunbundlepart(unbundlepart):
 
 # These are only the static capabilities.
 # Check the 'getrepocaps' function for the rest.
-capabilities = {
+capabilities: Capabilities = {
     b'HG20': (),
     b'bookmarks': (),
     b'error': (b'abort', b'unsupportedcontent', b'pushraced', b'pushkey'),
@@ -1626,7 +1656,8 @@ capabilities = {
 }
 
 
-def getrepocaps(repo, allowpushback=False, role=None):
+# TODO: drop the default value for 'role'
+def getrepocaps(repo, allowpushback: bool = False, role=None) -> Capabilities:
     """return the bundle2 capabilities for a given repo
 
     Exists to allow extensions (like evolution) to mutate the capabilities.
@@ -1675,7 +1706,7 @@ def getrepocaps(repo, allowpushback=False, role=None):
     return caps
 
 
-def bundle2caps(remote):
+def bundle2caps(remote) -> Capabilities:
     """return the bundle capabilities of a peer as dict"""
     raw = remote.capable(b'bundle2')
     if not raw and raw != b'':
@@ -1684,7 +1715,7 @@ def bundle2caps(remote):
     return decodecaps(capsblob)
 
 
-def obsmarkersversion(caps):
+def obsmarkersversion(caps: Capabilities):
     """extract the list of supported obsmarkers versions from a bundle2caps dict"""
     obscaps = caps.get(b'obsmarkers', ())
     return [int(c[1:]) for c in obscaps if c.startswith(b'V')]
@@ -1725,12 +1756,13 @@ def writenewbundle(
         msg %= count
         raise error.ProgrammingError(msg)
 
-    caps = {}
+    caps: Capabilities = {}
     if opts.get(b'obsolescence', False):
         caps[b'obsmarkers'] = (b'V1',)
-    if opts.get(b'streamv2'):
+    stream_version = opts.get(b'stream', b"")
+    if stream_version == b"v2":
         caps[b'stream'] = [b'v2']
-    elif opts.get(b'streamv3-exp'):
+    elif stream_version == b"v3-exp":
         caps[b'stream'] = [b'v3-exp']
     bundle = bundle20(ui, caps)
     bundle.setcompression(compression, compopts)
@@ -1774,10 +1806,10 @@ def _addpartsfromopts(ui, repo, bundler, source, outgoing, opts):
     if repository.REPO_FEATURE_SIDE_DATA in repo.features:
         part.addparam(b'exp-sidedata', b'1')
 
-    if opts.get(b'streamv2', False):
+    if opts.get(b'stream', b"") == b"v2":
         addpartbundlestream2(bundler, repo, stream=True)
 
-    if opts.get(b'streamv3-exp', False):
+    if opts.get(b'stream', b"") == b"v3-exp":
         addpartbundlestream2(bundler, repo, stream=True)
 
     if opts.get(b'tagsfnodescache', True):
@@ -1787,7 +1819,7 @@ def _addpartsfromopts(ui, repo, bundler, source, outgoing, opts):
         addpartrevbranchcache(repo, bundler, outgoing)
 
     if opts.get(b'obsolescence', False):
-        obsmarkers = repo.obsstore.relevantmarkers(outgoing.missing)
+        obsmarkers = repo.obsstore.relevantmarkers(nodes=outgoing.missing)
         buildobsmarkerspart(
             bundler,
             obsmarkers,
@@ -1844,10 +1876,8 @@ def addpartrevbranchcache(repo, bundler, outgoing):
             utf8branch = encoding.fromlocal(branch)
             yield rbcstruct.pack(len(utf8branch), len(nodes), len(closed))
             yield utf8branch
-            for n in sorted(nodes):
-                yield n
-            for n in sorted(closed):
-                yield n
+            yield from sorted(nodes)
+            yield from sorted(closed)
 
     bundler.newpart(b'cache:rev-branch-cache', data=generate(), mandatory=False)
 
@@ -2016,8 +2046,7 @@ def writebundle(
 
         def chunkiter():
             yield header
-            for chunk in compengine.compressstream(cg.getchunks(), compopts):
-                yield chunk
+            yield from compengine.compressstream(cg.getchunks(), compopts)
 
         chunkiter = chunkiter()
 
@@ -2100,7 +2129,7 @@ def handlechangegroup(op, inpart):
         op.source,
         b'bundle2',
         expectedtotal=nbchangesets,
-        **extrakwargs
+        **extrakwargs,
     )
     if op.reply is not None:
         # This is definitely not the final form of this
@@ -2597,7 +2626,6 @@ def bundle2getvars(op, part):
 
 @parthandler(b'stream2', (b'requirements', b'filecount', b'bytecount'))
 def handlestreamv2bundle(op, part):
-
     requirements = urlreq.unquote(part.params[b'requirements'])
     requirements = requirements.split(b',') if requirements else []
     filecount = int(part.params[b'filecount'])
