@@ -5,18 +5,19 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
+from __future__ import annotations
 
 import difflib
 import re
 import struct
+import typing
 
-from typing import (
-    List,
-    Tuple,
+from ..interfaces import (
+    modules as intmod,
 )
 
 
-def splitnewlines(text: bytes) -> List[bytes]:
+def splitnewlines(text: bytes) -> list[bytes]:
     '''like str.splitlines, but only split on newlines.'''
     lines = [l + b'\n' for l in text.split(b'\n')]
     if lines:
@@ -28,8 +29,8 @@ def splitnewlines(text: bytes) -> List[bytes]:
 
 
 def _normalizeblocks(
-    a: List[bytes], b: List[bytes], blocks
-) -> List[Tuple[int, int, int]]:
+    a: list[bytes], b: list[bytes], blocks
+) -> list[tuple[int, int, int]]:
     prev = None
     r = []
     for curr in blocks:
@@ -91,7 +92,7 @@ def bdiff(a: bytes, b: bytes) -> bytes:
     return b"".join(bin)
 
 
-def blocks(a: bytes, b: bytes) -> List[Tuple[int, int, int, int]]:
+def blocks(a: bytes, b: bytes) -> list[tuple[int, int, int, int]]:
     an = splitnewlines(a)
     bn = splitnewlines(b)
     d = difflib.SequenceMatcher(None, an, bn).get_matching_blocks()
@@ -106,3 +107,11 @@ def fixws(text: bytes, allws: bool) -> bytes:
         text = re.sub(b'[ \t\r]+', b' ', text)
         text = text.replace(b' \n', b'\n')
     return text
+
+
+# In order to adhere to the module protocol, these functions must be visible to
+# the type checker, though they aren't actually implemented by this
+# implementation of the module protocol.  Callers are responsible for
+# checking that the implementation is available before using them.
+if typing.TYPE_CHECKING:
+    xdiffblocks: intmod.BDiffBlocksFnc | None = None
