@@ -1,5 +1,4 @@
 # censor code related to censoring revision
-# coding: utf8
 #
 # Copyright 2021 Pierre-Yves David <pierre-yves.david@octobus.net>
 # Copyright 2015 Google, Inc <martinvonz@google.com>
@@ -7,6 +6,8 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
+from __future__ import annotations
+
 import binascii
 import contextlib
 import os
@@ -29,6 +30,8 @@ from .constants import (
     ENTRY_SIDEDATA_COMPRESSED_LENGTH,
     ENTRY_SIDEDATA_COMPRESSION_MODE,
     ENTRY_SIDEDATA_OFFSET,
+    META_MARKER,
+    META_MARKER_SIZE,
     REVIDX_ISCENSORED,
     REVLOGV0,
     REVLOGV1,
@@ -51,14 +54,14 @@ from . import (
 )
 
 
-def v1_censor(rl, tr, censornode, tombstone=b''):
+def v1_censor(rl, tr, censor_nodes, tombstone=b''):
     """censors a revision in a "version 1" revlog"""
     assert rl._format_version == constants.REVLOGV1, rl._format_version
 
     # avoid cycle
     from .. import revlog
 
-    censorrev = rl.rev(censornode)
+    censor_revs = {rl.rev(node) for node in censor_nodes}
     tombstone = storageutil.packmeta({b'censored': tombstone}, b'')
 
     # Rewriting the revlog in place is hard. Our strategy for censoring is
@@ -76,6 +79,7 @@ def v1_censor(rl, tr, censornode, tombstone=b''):
         delta_config=rl.delta_config,
         feature_config=rl.feature_config,
         may_inline=rl._inline,
+        writable=True,
     )
     # inline splitting will prepare some transaction work that will get
     # confused by the final file move. So if there is a risk of not being
@@ -87,14 +91,14 @@ def v1_censor(rl, tr, censornode, tombstone=b''):
         node = rl.node(rev)
         p1, p2 = rl.parents(node)
 
-        if rev == censorrev:
+        if rev in censor_revs:
             newrl.addrawrevision(
                 tombstone,
                 tr,
-                rl.linkrev(censorrev),
+                rl.linkrev(rev),
                 p1,
                 p2,
-                censornode,
+                node,
                 constants.REVIDX_ISCENSORED,
             )
 
@@ -134,16 +138,16 @@ def v1_censor(rl, tr, censornode, tombstone=b''):
         rl.opener.rename(newrl._datafile, rl._datafile)
 
     rl.clearcaches()
-    chunk_cache = rl._loadindex()
-    rl._load_inner(chunk_cache)
+    index, chunk_cache = rl._loadindex()
+    rl._load_inner(index, chunk_cache)
 
 
-def v2_censor(revlog, tr, censornode, tombstone=b''):
+def v2_censor(revlog, tr, censor_nodes, tombstone=b''):
     """censors a revision in a "version 2" revlog"""
     assert revlog._format_version != REVLOGV0, revlog._format_version
     assert revlog._format_version != REVLOGV1, revlog._format_version
 
-    censor_revs = {revlog.rev(censornode)}
+    censor_revs = {revlog.rev(node) for node in censor_nodes}
     _rewrite_v2(revlog, tr, censor_revs, tombstone)
 
 
@@ -258,13 +262,12 @@ def _precompute_rewritten_delta(
                 # this revision is empty, we can delta against nullrev
                 rewritten_entries[rev] = (nullrev, 0, 0, COMP_MODE_PLAIN)
             else:
-
                 text = revlog.rawdata(rev)
                 info = revlogutils.revisioninfo(
                     node=entry[ENTRY_NODE_ID],
                     p1=revlog.node(entry[ENTRY_PARENT_1]),
                     p2=revlog.node(entry[ENTRY_PARENT_2]),
-                    btext=[text],
+                    btext=text,
                     textlen=len(text),
                     cachedelta=None,
                     flags=entry[ENTRY_DATA_OFFSET] & 0xFFFF,
@@ -326,7 +329,8 @@ def _setup_new_files(
 
     # reload the revlog internal information
     revlog.clearcaches()
-    revlog._loadindex(docket=docket)
+    index, chunk_cache = revlog._loadindex(docket=docket)
+    revlog._load_inner(index, chunk_cache)
 
     @contextlib.contextmanager
     def all_files_opener():
@@ -498,15 +502,6 @@ def _get_filename_from_filelog_index(path):
     return path_part[1]
 
 
-def _filelog_from_filename(repo, path):
-    """Returns the filelog for the given `path`. Stolen from `engine.py`"""
-
-    from .. import filelog  # avoid cycle
-
-    fl = filelog.filelog(repo.svfs, path)
-    return fl
-
-
 def _write_swapped_parents(repo, rl, rev, offset, fp):
     """Swaps p1 and p2 and overwrites the revlog entry for `rev` in `fp`"""
     from ..pure import parsers  # avoid cycle
@@ -525,6 +520,37 @@ def _write_swapped_parents(repo, rl, rev, offset, fp):
     fp.write(packed)
 
 
+### Constant used for `delta_has_meta` function
+#
+# Can't know if the revision has meta from this delta
+HM_UNKNOWN = -2
+# We know the revision is similar to the delta parent
+HM_INHERIT = -1
+# We know the revision  does not have meta
+HM_NO_META = 0
+# we know the revision has meta information
+HM_META = 1
+
+
+def delta_has_meta(delta: bytes, has_base: bool = True) -> int:
+    """determine if a revision has metadata from its delta"""
+    if not has_base:
+        if delta[:META_MARKER_SIZE] == META_MARKER:
+            return HM_META
+        else:
+            return HM_NO_META
+    before, local_bytes, after = mdiff.first_bytes(delta, META_MARKER_SIZE)
+    if before + after == META_MARKER_SIZE:
+        return HM_INHERIT
+    elif len(local_bytes) == META_MARKER_SIZE:
+        if local_bytes == META_MARKER:
+            return HM_META
+        else:
+            return HM_NO_META
+    else:
+        return HM_UNKNOWN
+
+
 def _reorder_filelog_parents(repo, fl, to_fix):
     """
     Swaps p1 and p2 for all `to_fix` revisions of filelog `fl` and writes the
@@ -568,7 +594,8 @@ def _reorder_filelog_parents(repo, fl, to_fix):
 
             rl.opener.rename(new_file_path, index_file)
             rl.clearcaches()
-            rl._loadindex()
+            index, chunk_cache = rl._loadindex()
+            rl._load_inner(index, chunk_cache)
         finally:
             util.tryunlink(new_file_path)
 
@@ -600,7 +627,7 @@ def _is_revision_affected_inner(
         return False
 
     # raw text can be a `memoryview`, which doesn't implement `startswith`
-    has_meta = bytes(raw_text[:2]) == b'\x01\n'
+    has_meta = bytes(raw_text[:META_MARKER_SIZE]) == META_MARKER
     if metadata_cache is not None:
         metadata_cache[filerev] = has_meta
     if has_meta:
@@ -614,15 +641,20 @@ def _is_revision_affected_fast(repo, fl, filerev, metadata_cache):
     rl = fl._revlog
     is_censored = lambda: rl.iscensored(filerev)
     delta_base = lambda: rl.deltaparent(filerev)
-    delta = lambda: rl._chunk(filerev)
+    delta = lambda: rl._inner._chunk(filerev)
     full_text = lambda: rl.rawdata(filerev)
     parent_revs = lambda: rl.parentrevs(filerev)
+    # This function is used by repair_issue6528, but not by
+    # filter_delta_issue6528. As such, we do not want to trust
+    # parent revisions of the delta base to decide whether
+    # the delta base has metadata.
     return _is_revision_affected_fast_inner(
         is_censored,
         delta_base,
         delta,
         full_text,
         parent_revs,
+        None,  # don't trust the parent revisions
         filerev,
         metadata_cache,
     )
@@ -634,6 +666,7 @@ def _is_revision_affected_fast_inner(
     delta,
     full_text,
     parent_revs,
+    deltabase_parentrevs,
     filerev,
     metadata_cache,
 ):
@@ -652,21 +685,36 @@ def _is_revision_affected_fast_inner(
 
     p1, p2 = parent_revs()
     if p1 == nullrev or p2 != nullrev:
+        metadata_cache[filerev] = True
         return False
 
     delta_parent = delta_base()
     parent_has_metadata = metadata_cache.get(delta_parent)
     if parent_has_metadata is None:
-        return _is_revision_affected_inner(
-            full_text,
-            parent_revs,
-            filerev,
-            metadata_cache,
-        )
+        if deltabase_parentrevs is not None:
+            deltabase_parentrevs = deltabase_parentrevs()
+            if deltabase_parentrevs == (nullrev, nullrev):
+                # Need to check the content itself as there is no flag.
+                parent_has_metadata = None
+            elif deltabase_parentrevs[0] == nullrev:
+                # Second parent is !null, assume repository is correct
+                # and has flagged this file revision as having metadata.
+                parent_has_metadata = True
+            elif deltabase_parentrevs[1] == nullrev:
+                # First parent is !null, so assume it has no metadata.
+                parent_has_metadata = False
+        if parent_has_metadata is None:
+            return _is_revision_affected_inner(
+                full_text,
+                parent_revs,
+                filerev,
+                metadata_cache,
+            )
 
     chunk = delta()
     if not len(chunk):
         # No diff for this revision
+        metadata_cache[filerev] = parent_has_metadata
         return parent_has_metadata
 
     header_length = 12
@@ -675,7 +723,7 @@ def _is_revision_affected_fast_inner(
 
     start, _end, _length = struct.unpack(b">lll", chunk[:header_length])
 
-    if start < 2:  # len(b'\x01\n') == 2
+    if start < META_MARKER_SIZE:
         # This delta does *something* to the metadata marker (if any).
         # Check it the slow way
         is_affected = _is_revision_affected_inner(
@@ -707,10 +755,10 @@ def _from_report(ui, repo, context, from_report, dry_run):
             if not line:
                 continue
             filenodes, filename = line.split(b' ', 1)
-            fl = _filelog_from_filename(repo, filename)
-            to_fix = set(
+            fl = repo.file(filename, writable=True)
+            to_fix = {
                 fl.rev(binascii.unhexlify(n)) for n in filenodes.split(b',')
-            )
+            }
             excluded = set()
 
             for filerev in to_fix:
@@ -734,77 +782,56 @@ def _from_report(ui, repo, context, from_report, dry_run):
 
 def filter_delta_issue6528(revlog, deltas_iter):
     """filter incomind deltas to repaire issue 6528 on the fly"""
-    metadata_cache = {}
+    metadata_cache = {nullrev: False}
 
     deltacomputer = deltas.deltacomputer(revlog)
 
     for rev, d in enumerate(deltas_iter, len(revlog)):
-        (
-            node,
-            p1_node,
-            p2_node,
-            linknode,
-            deltabase,
-            delta,
-            flags,
-            sidedata,
-        ) = d
-
-        if not revlog.index.has_node(deltabase):
+        if not revlog.index.has_node(d.delta_base):
             raise error.LookupError(
-                deltabase, revlog.radix, _(b'unknown parent')
+                d.delta_base, revlog.radix, _(b'unknown parent')
             )
-        base_rev = revlog.rev(deltabase)
-        if not revlog.index.has_node(p1_node):
-            raise error.LookupError(p1_node, revlog.radix, _(b'unknown parent'))
-        p1_rev = revlog.rev(p1_node)
-        if not revlog.index.has_node(p2_node):
-            raise error.LookupError(p2_node, revlog.radix, _(b'unknown parent'))
-        p2_rev = revlog.rev(p2_node)
+        base_rev = revlog.rev(d.delta_base)
+        if not revlog.index.has_node(d.p1):
+            raise error.LookupError(d.p1, revlog.radix, _(b'unknown parent'))
+        p1_rev = revlog.rev(d.p1)
+        if not revlog.index.has_node(d.p2):
+            raise error.LookupError(d.p2, revlog.radix, _(b'unknown parent'))
+        p2_rev = revlog.rev(d.p2)
 
-        is_censored = lambda: bool(flags & REVIDX_ISCENSORED)
-        delta_base = lambda: revlog.rev(delta_base)
+        is_censored = lambda: bool(d.flags & REVIDX_ISCENSORED)
         delta_base = lambda: base_rev
         parent_revs = lambda: (p1_rev, p2_rev)
+        deltabase_parentrevs = lambda: revlog.parentrevs(base_rev)
 
         def full_text():
-            # note: being able to reuse the full text computation in the
-            # underlying addrevision would be useful however this is a bit too
-            # intrusive the for the "quick" issue6528 we are writing before the
-            # 5.8 release
-            textlen = mdiff.patchedsize(revlog.size(base_rev), delta)
+            if d.raw_text is None:
+                textlen = mdiff.patchedsize(revlog.size(base_rev), d.delta)
 
-            revinfo = revlogutils.revisioninfo(
-                node,
-                p1_node,
-                p2_node,
-                [None],
-                textlen,
-                (base_rev, delta),
-                flags,
-            )
-            return deltacomputer.buildtext(revinfo)
+                revinfo = revlogutils.revisioninfo(
+                    d.node,
+                    d.p1,
+                    d.p2,
+                    None,
+                    textlen,
+                    revlogutils.CachedDelta(base_rev, d.delta),
+                    d.flags,
+                )
+                d.raw_text = deltacomputer.buildtext(revinfo)
+            return d.raw_text
 
         is_affected = _is_revision_affected_fast_inner(
             is_censored,
             delta_base,
-            lambda: delta,
+            lambda: d.delta,
             full_text,
             parent_revs,
+            deltabase_parentrevs,
             rev,
             metadata_cache,
         )
         if is_affected:
-            d = (
-                node,
-                p2_node,
-                p1_node,
-                linknode,
-                deltabase,
-                delta,
-                flags,
-                sidedata,
-            )
+            d.p2, d.p1 = d.p1, d.p2
         yield d
 
 
@@ -841,11 +868,11 @@ def repair_issue6528(
         for entry in files:
             progress.increment()
             filename = entry.target_id
-            fl = _filelog_from_filename(repo, entry.target_id)
+            fl = repo.file(entry.target_id, writable=not dry_run)
 
             # Set of filerevs (or hex filenodes if `to_report`) that need fixing
             to_fix = set()
-            metadata_cache = {}
+            metadata_cache = {nullrev: False}
             for filerev in fl.revs():
                 affected = _is_revision_affected_fast(
                     repo, fl, filerev, metadata_cache
@@ -882,3 +909,164 @@ def repair_issue6528(
                     f.write(b"%s %s\n" % (b",".join(to_fix), path))
 
         progress.complete()
+
+
+def _find_all_revs_with_meta(rl):
+    meta = {}
+    for filerev in rl:
+        if rl.iscensored(filerev):
+            continue
+        delta_parent = rl.deltaparent(filerev)
+        has_base = delta_parent >= 0
+
+        delta = rl._inner._chunk(filerev)
+
+        hm = delta_has_meta(delta, has_base)
+        if hm == HM_META:
+            meta[filerev] = True
+        elif hm == HM_NO_META:
+            meta[filerev] = False
+        elif hm == HM_INHERIT and delta_parent in meta:
+            meta[filerev] = meta[delta_parent]
+        else:
+            try:
+                revdata = rl._revisiondata(
+                    filerev,
+                    validate=False,
+                )
+            except error.CensoredNodeError:
+                meta[filerev] = False
+            else:
+                meta[filerev] = revdata[:META_MARKER_SIZE] == META_MARKER
+
+    return {k for k, v in meta.items() if v}
+
+
+def _find_all_snapshots(rl):
+    snapshots = set()
+    if hasattr(rl.index, 'findsnapshots'):
+        cache: dict[int, set[int]] = {}
+        rl.index.findsnapshots(cache, 0, len(rl))
+        for b, c in cache.items():
+            snapshots.add(b)
+            snapshots.update(c)
+    else:
+        for rev in rl:
+            if rl.issnapshot(rev):
+                snapshots.add(rev)
+    return snapshots
+
+
+def quick_upgrade(rl, upgrade_meta, upgrade_delta_info):
+    assert rl._format_flags & constants.FLAG_GENERALDELTA
+
+    if rl.target[0] != constants.KIND_FILELOG:
+        upgrade_meta = False
+
+    if upgrade_meta and rl._format_flags & constants.FLAG_FILELOG_META:
+        upgrade_meta = False
+
+    if rl.target[0] not in (constants.KIND_FILELOG, constants.KIND_MANIFESTLOG):
+        upgrade_delta_info = False
+
+    if upgrade_delta_info and rl._format_flags & constants.FLAG_DELTA_INFO:
+        upgrade_delta_info = False
+
+    if not (upgrade_meta or upgrade_delta_info):
+        return
+
+    new_flags = 0
+
+    if upgrade_meta:
+        revs_with_meta = _find_all_revs_with_meta(rl)
+        new_flags |= constants.FLAG_FILELOG_META
+    else:
+        revs_with_meta = set()
+
+    if upgrade_delta_info:
+        revs_with_snapshot = _find_all_snapshots(rl)
+        new_flags |= constants.FLAG_DELTA_INFO
+    else:
+        revs_with_snapshot = set()
+
+    if not (revs_with_meta or revs_with_snapshot):
+        # We just need to write the header flag
+        # XXX do we need to sort the parent anyway?
+        with rl.opener(rl._indexfile, b'br+') as n:
+            n.seek(0)
+            first_entry = rl.index.entry_binary(0)
+            header = rl._format_flags
+            header |= rl._format_version
+            header |= new_flags
+            header = rl.index.pack_header(header)
+            first_entry = header + first_entry
+            n.write(first_entry)
+        return
+
+    index = rl.index
+    new_index = rl._parse_index(
+        b'',
+        False,
+        True,
+        True,
+    )[0]
+
+    for filerev in rl:
+        (
+            data_offset_and_flag,
+            data_compressed_length,
+            data_uncompressed_length,
+            delta_base,
+            link_rev,
+            parent_1,
+            parent_2,
+            node_id,
+            side_data_offset,
+            side_data_compressed_length,
+            data_compression_mode,
+            sidedata_compression_mode,
+            rank,
+        ) = index[filerev]
+        flags = data_offset_and_flag & 0xFFFF
+        dataoffset = int(data_offset_and_flag >> 16)
+
+        if parent_1 == nullrev and parent_2 != nullrev:
+            parent_1, parent_2 = parent_2, parent_1
+
+        if filerev in revs_with_meta:
+            flags |= constants.REVIDX_HASMETA
+
+        if filerev in revs_with_snapshot:
+            flags |= constants.REVIDX_DELTA_IS_SNAPSHOT
+
+        e = revlogutils.entry(
+            flags=flags,
+            data_offset=dataoffset,
+            data_compressed_length=data_compressed_length,
+            data_uncompressed_length=data_uncompressed_length,
+            data_compression_mode=data_compression_mode,
+            data_delta_base=delta_base,
+            link_rev=link_rev,
+            parent_rev_1=parent_1,
+            parent_rev_2=parent_2,
+            node_id=node_id,
+            sidedata_offset=side_data_offset,
+            sidedata_compressed_length=side_data_compressed_length,
+            sidedata_compression_mode=sidedata_compression_mode,
+            rank=rank,
+        )
+        new_index.append(e)
+
+    # write data
+    with rl.opener(rl._indexfile, b'wb', atomictemp=True) as n:
+        for rev in range(len(new_index)):
+            idx = new_index.entry_binary(rev)
+            if rev == 0:
+                header = rl._format_flags
+                header |= rl._format_version
+                header |= new_flags
+                header = new_index.pack_header(header)
+                idx = header + idx
+            n.write(idx)
+            if rl._inline:
+                n.write(rl._inner.get_segment_for_revs(rev, rev)[1])
