@@ -65,14 +65,18 @@ mappedgenerator
     operation.
 """
 
+from __future__ import annotations
 
 import abc
 import os
 
+from typing import (
+    IO,
+)
+
 from .i18n import _
 from .pycompat import (
     FileNotFoundError,
-    getattr,
 )
 from . import (
     config,
@@ -956,12 +960,12 @@ class loader:
                 raise templateutil.TemplateNotFound(
                     _(b'"%s" not in template map') % inst.args[0]
                 )
-            except IOError as inst:
+            except OSError as inst:
                 reason = _(b'template file %s: %s') % (
                     self._map[t],
                     stringutil.forcebytestr(inst.args[1]),
                 )
-                raise IOError(inst.args[0], encoding.strfromlocal(reason))
+                raise OSError(inst.args[0], encoding.strfromlocal(reason))
         return self._parse(self.cache[t])
 
     def _parse(self, tmpl):
@@ -1122,7 +1126,9 @@ def templatedir():
     return path if os.path.isdir(path) else None
 
 
-def open_template(name, templatepath=None):
+def open_template(
+    name: bytes, templatepath: bytes | None = None
+) -> tuple[bytes, IO[bytes]]:
     """returns a file-like object for the given template, and its full path
 
     If the name is a relative path and we're in a frozen binary, the template
@@ -1157,8 +1163,10 @@ def open_template(name, templatepath=None):
     )
 
 
-def try_open_template(name, templatepath=None):
+def try_open_template(
+    name: bytes, templatepath: bytes | None = None
+) -> tuple[bytes | None, IO[bytes] | None]:
     try:
         return open_template(name, templatepath)
-    except (EnvironmentError, ImportError):
+    except (OSError, ImportError):
         return None, None
